require 'open-uri'
require 'seven_zip_ruby'
require 'rubygems/package'
require 'zlib'

module OS
    def OS.windows?
      (/cygwin|mswin|mingw|bccwin|wince|emx/ =~ RUBY_PLATFORM) != nil
    end
  
    def OS.mac?
     (/darwin/ =~ RUBY_PLATFORM) != nil
    end
  
    def OS.unix?
      !OS.windows?
    end
  
    def OS.linux?
      OS.unix? and not OS.mac?
    end
  
    def OS.jruby?
      RUBY_ENGINE == 'jruby'
    end
end

namespace :prepare do
    task sgf_render: [ "sgf_render:extract" ] do |t|
    end

    namespace :sgf_render do
        task extract: [ "extract:windows", "extract:linux", "extract:macosx" ] do
            puts("SGF-Render is available at #{TOOLS_BIN_DIR}:")
        end

        namespace :extract do
            windows_executable = File.join(TOOLS_BIN_DIR, "sgf-render.exe")
            unix_executable = File.join(TOOLS_BIN_DIR, "sgf-render")
        
            task :dir do
                Dir.mkdir(TMP_DIR) unless Dir.exists?(TMP_DIR)
                Dir.mkdir(TOOLS_DIR) unless Dir.exists?(TOOLS_DIR)
                Dir.mkdir(TOOLS_BIN_DIR) unless Dir.exists?(TOOLS_BIN_DIR)
            end

            task windows: [ "dir", windows_executable ] do
                tools_bin_dir = Pathname.new(TOOLS_BIN_DIR).realpath
                ENV["PATH"] = "#{ENV["PATH"]};#{tools_bin_dir}" unless ENV['PATH'].include?(TOOLS_BIN_DIR)
            end

            file windows_executable do
                if OS.windows?
                    download = open("https://github.com/julianandrews/sgf-render/releases/download/v#{SFG_RENDER_VERSION}/sgf-render-windows.7z")
                    x_TARGET_FILE = File.join(TMP_DIR, "sgf-render-windows.7z")
                    IO.copy_stream(download, x_TARGET_FILE)
                    File.open(TARGET_FILE, "rb") do |file|
                        SevenZipRuby::Reader.open(file) do |szr|
                            szr.extract_all TOOLS_BIN_DIR
                        end
                    end
                end
            end

            task linux: [ "dir", unix_executable ] do
                File.chmod(0755, unix_executable)
                ENV["PATH"] = "#{ENV["PATH"]}:#{TOOLS_BIN_DIR}" unless ENV['PATH'].include?(TOOLS_BIN_DIR)
            end

            task macosx: [ "dir", unix_executable ] do
                File.chmod(0755, unix_executable)
                ENV["PATH"] = "#{ENV["PATH"]}:#{TOOLS_BIN_DIR}" unless ENV['PATH'].include?(TOOLS_BIN_DIR)
            end

            file unix_executable do
                x_TARGET_FILE = nil
                begin
                    if OS.mac?
                        download = URI.open("https://github.com/julianandrews/sgf-render/releases/download/v#{SFG_RENDER_VERSION}/sgf-render-macos.tar.gz")
                        x_TARGET_FILE = File.join(TMP_DIR, "sgf-render-macos.tar.gz")
                        IO.copy_stream(download, x_TARGET_FILE)
                    elsif OS.linux?
                        download = URI.open("https://github.com/julianandrews/sgf-render/releases/download/v#{SFG_RENDER_VERSION}/sgf-render-linux.tar.gz")
                        x_TARGET_FILE = File.join(TMP_DIR, "sgf-render-linux.tar.gz")
                        IO.copy_stream(download, x_TARGET_FILE)
                    else 
                        puts("Failed to download archive")
                    end

                    if not x_TARGET_FILE.nil?
                        tar_extract = Gem::Package::TarReader.new(Zlib::GzipReader.open(x_TARGET_FILE))
                        tar_extract.rewind 
                        tar_extract.each do |entry|
                            dest =  File.join(TOOLS_BIN_DIR, entry.full_name)
                            if entry.file?
                                File.open(dest, "wb") do |f|
                                    f.print(entry.read())
                                end
                            end
                        end
                        tar_extract.close
                    end
                rescue
                    puts("#{$!}")
                ensure
                    if not x_TARGET_FILE.nil?
                        File.delete(x_TARGET_FILE)
                    end
                end
            end
        end
    end
end