require 'yaml'
require 'pathname'
require 'fileutils'

namespace :sgf_render do
    assets_dir = File.join(TARGET_DIRECTORY, "assets")
    assets_figure_dir = File.join(assets_dir, "figures")
    SGF_GOBANS_DIR = File.join(".", "gobans")

    SGF_FILES = Rake::FileList.new(File.join(SGF_GOBANS_DIR, "*.sgf"))
    
    task :dir do
        Dir.mkdir(TARGET_DIRECTORY) unless Dir.exists?(TARGET_DIRECTORY) 
        Dir.mkdir(assets_dir) unless Dir.exists?(assets_dir) 
        Dir.mkdir(assets_figure_dir) unless Dir.exists?(assets_figure_dir)
    end

    task :figures => [ :render, :copy ]

    task :render => SGF_FILES.ext(".png")

    task :copy => [ :dir, :render ] do
        target_assets_figure_dir = Pathname.new(assets_figure_dir).realpath
        Dir.chdir(SGF_GOBANS_DIR) do
            Dir.glob("*.png").each do |file|
                FileUtils.cp(file, target_assets_figure_dir)
            end
        end
    end

    rule ".png" => ".sgf" do |t|
        node = get_node_count t.name

        sh "sgf-render #{t.source} -n #{node} --style #{GOBAN_STYLE} -o #{t.name} --no-board-labels --no-marks"
    end

    def get_node_count(png_file)
        base = File.basename png_file.ext("")
        if GOBAN_NODES.include? base
            GOBAN_NODES[base]
        else
            0
        end
    end
end